import _isObject from "lodash/isObject";
import _clamp from "lodash/clamp";
import { Canvas } from "@tarojs/components";
import { createCanvasContext, getSystemInfoSync } from "@tarojs/taro";
import * as React from "react";
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
import { useMounted, useUniqueId } from "../hooks";
import { BLUE, WHITE } from "../styles/variables";
import { queryNodesRef } from "../utils/dom/element";
import { addUnitPx } from "../utils/format/unit";
import { canIUseCanvas2d } from "../utils/version";
import { canvasContextAdaptor } from "./canvas";
import { useAnimatePercent } from "./circle.hooks";

function clampFormat(rate) {
  return _clamp(rate, 0, 100);
}

var PERIMETER = 2 * Math.PI;
var BEGIN_ANGLE = -Math.PI / 2;

function useCanvasContext(canvasRef, size) {
  var [canvasContext, setCanvasContext] = useState();
  useMounted(() => {
    if (!canIUseCanvas2d()) {
      if (canvasRef.current) {
        setCanvasContext(createCanvasContext(canvasRef.current.id));
      }
    } else {
      if (canvasRef.current) {
        queryNodesRef(canvasRef.current).fields({
          node: true
        }, _ref => {
          var {
            node: canvas
          } = _ref;
          var {
            pixelRatio: dpr
          } = getSystemInfoSync();
          canvas.width = size * dpr;
          canvas.height = size * dpr;
          var newCanvasContext = canvasContextAdaptor(canvas.getContext("2d"));
          newCanvasContext.scale(dpr, dpr);
          setCanvasContext(newCanvasContext);
        }).exec();
      }
    }
  });
  return canvasContext;
}

function CircleCanvas(props) {
  var {
    percent: percentProp = 0,
    speed = 0,
    color = BLUE,
    layerColor = WHITE,
    fill,
    clockwise,
    strokeWidth: strokeWidthProp = 0,
    strokeLinecap,
    size = 100,
    onChange
  } = props;
  var canvasId = useUniqueId();
  var canvasRef = useRef();
  var strokeWidth = useMemo(() => strokeWidthProp / 10, [strokeWidthProp]);
  var percent = useAnimatePercent(percentProp, speed);
  var canvasContext = useCanvasContext(canvasRef, size);
  var [hoverColor, setHoverColor] = useState();
  var presetCanvas = useCallback((context, strokeStyle, beginAngle, endAngle, fill) => {
    var position = size / 2;
    var radius = position - strokeWidth / 2;
    context.setStrokeStyle(strokeStyle);
    context.setLineWidth(strokeWidth);
    context.setLineCap(strokeLinecap);
    context.beginPath();
    context.arc(position, position, radius, beginAngle, endAngle, !clockwise);
    context.stroke();

    if (fill) {
      context.setFillStyle(fill);
      context.fill();
    }
  }, [clockwise, size, strokeLinecap, strokeWidth]);
  var renderLayerCircle = useCallback(context => {
    presetCanvas(context, layerColor, 0, PERIMETER, fill);
  }, [fill, layerColor, presetCanvas]);
  var renderHoverCircle = useCallback((context, formatValue) => {
    var progress = PERIMETER * (formatValue / 100);
    var endAngle = clockwise ? BEGIN_ANGLE + progress : 3 * Math.PI - (BEGIN_ANGLE + progress);
    presetCanvas(context, hoverColor, BEGIN_ANGLE, endAngle);
  }, [clockwise, hoverColor, presetCanvas]);
  var drawCircle = useCallback(currentValue => {
    if (canvasContext) {
      canvasContext.clearRect(0, 0, size, size);
      renderLayerCircle(canvasContext);
      var formatValue = clampFormat(currentValue);

      if (formatValue !== 0) {
        renderHoverCircle(canvasContext, formatValue);
      }

      canvasContext.draw();
    }
  }, [canvasContext, renderHoverCircle, renderLayerCircle, size]);
  useEffect(() => drawCircle(percent), [drawCircle, percent]);
  useEffect(() => onChange === null || onChange === void 0 ? void 0 : onChange(percent), [percent]);
  useEffect(() => {
    if (_isObject(color)) {
      var LinearColor = canvasContext === null || canvasContext === void 0 ? void 0 : canvasContext.createLinearGradient(size, 0, 0, 0);

      if (LinearColor) {
        Object.keys(color).sort((a, b) => parseFloat(a) - parseFloat(b)).forEach(key => LinearColor.addColorStop(parseFloat(key) / 100, color[key]));
        setHoverColor(LinearColor);
      }
    } else {
      setHoverColor(color);
    }
  }, [canvasContext, color, size]);
  return /*#__PURE__*/React.createElement(Canvas, {
    ref: canvasRef,
    id: canvasId,
    canvasId: canvasId,
    type: "2d",
    style: {
      width: addUnitPx(size),
      height: addUnitPx(size)
    }
  });
}

export default CircleCanvas;
//# sourceMappingURL=circle-canvas.js.map
